#include <string.h>
#include "geosim.h"
#include "gspriv.h"
#include "miscwind.h"
#include "ietypes.h"
#include "iefile.h"
#include "windfunc.h"
/*#include "rootw.h"*/
#include "iemisc.h"
#include "editrec.h"
#include "aliasw.h"
#include "helpmsg.h"

#define MAX_SPEEDUP 201
#define F_HIGHLIGHT 0
#define F_BORDER 1
#define F_TL 2
#define F_BR 3
#define F_ALL 4

boolean numeric_filter(char *);

boolean boolean_filter(char *);
/*void diag_notify();*/

extern BigString filename;
extern boolean miscwind;
extern IEIconfig iestate;
extern char *IEColorlist[], *IEAlias[];

BigString speedup_text, diag_text;

static EditRec *redrawfunc_rec;
static GSsclistHandle colorScroll;
static GSlistHandle colorList;
static int numListColors, numColors, curColor;

/* Internal Protos */
void ShowSpeedup();
void InitButtons();
void MSetupColorList();
void EnableButtonStuff();
void DisableButtonStuff();
void ShowButtonStuffFields(int);
char *GetColor(int);
/**/

static void textFunc(GSlistHandle list, int slot)
{	
	if ((list == colorList)&&(slot == curColor))
		
		GSsetcolor(GSconvertcolor("red"));
	else
		GSsetcolor(GSconvertcolor("black"));
	GSsetfont(SMALL);
}

static char *cLabelFunc(int slot)
{
	if (slot < numListColors)
		return IEColorlist[slot];
	else if (slot < numColors)
		return IEAlias[slot-numListColors];
	else	
		return "";
}

static void cInFunc() 
{
}

static void cOutFunc() 
{
}

static void cClickFunc(int slot)
{
	int old = curColor;
	
	curColor = slot;
	
	if (old == -1)
		EnableButtonStuff();
	
	GSdrawlistitem(colorList, New, old);
	GSdrawlistitem(colorList, New, curColor);
}

void RedrawMiscWind()
{
	int x, y, wd, ht;
	
	GSredrawnamedwindow("Misc");
	DrawEditRec(redrawfunc_rec);
	
	ShowSpeedup();
	ShowButtonStuffFields(F_ALL);
	GSdrawsclist(colorScroll);
	GSgetdragareaspecs("Misc", "colpickrect", &x, &y, 
								&wd, &ht, NULL, NULL);
	ShadowRect(x, y, wd, ht, THIN, Concave);
}

void DrawMiscWind()
{
	IEMessage *msg;
	
	InitButtons();
	GSdrawnamedwindow("Misc", NO_BSTORE, NO_POPUP);
	miscwind = TRUE;
	
/*	sprintf(speedup_text, "%d", iestate.speedup);*/

	redrawfunc_rec = NewEditRec(iestate.redrawfunc, BIGSTRINGLEN, 1, 
			SMALL, "Misc", "redrawfuncsel", NULL, NULL);

	DrawEditRec(redrawfunc_rec);
	SetCurrentEditRec(redrawfunc_rec);
	
	ShowSpeedup();
	/*ShowButtonStuffFields(F_ALL);*/
	DisableButtonStuff();
	
	GSdisablemenuitem("Edit", "edit_misc");
	
	ClearDragRect("Misc", "colpickrect");
	
	MSetupColorList();
	
	GSdisplayhelpmessage(MISC_H);
	ShowVersion();
}

void ShowSpeedup()
{
	DrawNumField("speedupsel", iestate.speedup);
}

void RemoveMiscWind()
{
	GSremovenamedwindow("Misc");
	miscwind = FALSE;
	DisposEditRec(redrawfunc_rec);
	GSredrawnamedwindow("root");
	GSredrawnamedwindow("message");
	GSenablemenuitem("Edit", "edit_misc");

	GSdismantlesclist(colorScroll);
	curColor = -1;
}

static void MSetupColorList()
{
	int x, y, wd, dragHt;

	for (numListColors = 0; IEColorlist[numListColors] != NULL; numListColors++);
	for (numColors = numListColors; 
							IEAlias[numColors-numListColors] != NULL; numColors++);

	GSgetdragareaspecs("Misc", "colpickrect", &x, &y, 
								&wd, &dragHt, NULL, NULL);
	curColor = NO_ITEM;
	
	GSsclistsetup("Misc", "colpickrect",
					1,
					"picklabel",
					0, 0,
					JUST_LEFT,
					cLabelFunc,
					textFunc, 
					NULL,
					NULL,
					cClickFunc,
					"colpickscroll",
					GSconvertcolor("sclistcol"),
					GSconvertcolor("sclisttl"),
					GSconvertcolor("sclistbr"),
					numColors, 0, 
					NULL,
					"CUpArrow", "CDownArrow", 
					&colorList, &colorScroll);
							
					
	ShadowRect(x, y, wd, dragHt, THIN, Concave);
	GSdrawsclist(colorScroll);
}

void EnableButtonStuff()
{
	/* Called after we have a current color. */
	
	GSenablebutton("Misc", "highlightset", TRUE);
	GSenablebutton("Misc", "borderset", TRUE);
	GSenablebutton("Misc", "tlset", TRUE);
	GSenablebutton("Misc", "brset", TRUE);
	/* others here */
	ShowButtonStuffFields(F_ALL);
}

void DisableButtonStuff()
{
	GSdisablebutton("Misc", "highlightset", TRUE);
	GSdisablebutton("Misc", "borderset", TRUE);
	GSdisablebutton("Misc", "tlset", TRUE);
	GSdisablebutton("Misc", "brset", TRUE);

	ShowButtonStuffFields(F_ALL);
}

void ShowButtonStuffFields(int which)
{
	int x, y, wd, ht;
	GSColor col;
	char *names[F_ALL][2] = {NULL, "highlightdisp",
							NULL, "borderdisp",
							NULL, "tldisp",
							NULL, "brdisp"};
	int i;
	
	names[F_HIGHLIGHT][0] = iestate.highlight;
	names[F_BORDER][0] = iestate.buttonborder;
	names[F_TL][0] = iestate.buttonTLcolor;
	names[F_BR][0] = iestate.buttonBRcolor;
	
	for (i=0; i<F_ALL; i++)
		if ((which==F_ALL)||(which==i)) {
			GSgetfieldrect(names[i][1], &x, &y, &wd, &ht, &col);
			GSsetcolor(GSconvertcolor("white"));
			GSwfillrect(x, y, wd, ht);
			GSsetlinesize(1);
			GSframefield("Misc", names[i][1], 
				GSconvertcolor("black"), GSconvertcolor("dgray"));
			GSsetcolor(col);
			GSwwritemsg(x, y+SMALL, names[i][0]);
			}
}

/*void diag_notify()
{
	sscanf(diag_text, "%d", &iestate.diagonals);
}
*/
boolean misc_sel_redrawfunc(int x, int y, DragStatusType status)
{
	if (status == MOUSE_CLICK)
		SetCurrentEditRec(redrawfunc_rec);
}

boolean misc_inc_speedup()
{
	iestate.speedup++;
	
	if (iestate.speedup >= MAX_SPEEDUP)
		GSdisablebutton("Misc", "speedupinc", TRUE);
		
	if (iestate.speedup == 1)
		GSenablebutton("Misc", "speedupdec", TRUE);

	ShowSpeedup();
}

boolean misc_dec_speedup()
{
	if (iestate.speedup > 0)
		iestate.speedup--;

	if (iestate.speedup <= 0) 
		GSdisablebutton("Misc", "speedupdec", TRUE);

	if (iestate.speedup == MAX_SPEEDUP - 1)
		GSenablebutton("Misc", "speedupinc", TRUE);

	ShowSpeedup();
}

boolean misc_toggle_diag()
{
	iestate.diagonals = !iestate.diagonals;
	
	GSsetbuttontext("Misc", "togglebdiag", iestate.diagonals? "Yes" : "No");
	
	GSenablebutton("Misc", "togglebdiag", TRUE);
}

static void InitButtons()
{
	GSsetbuttontext("Misc", "togglebdiag", iestate.diagonals? "Yes" : "No");

	if (iestate.speedup > 0)
		GSenablebutton("Misc", "speedupdec", FALSE);
	else
		GSdisablebutton("Misc", "speedupdec", FALSE);

	if (iestate.speedup < MAX_SPEEDUP)
		GSenablebutton("Misc", "speedupinc", FALSE);
	else
		GSdisablebutton("Misc", "speedupinc", FALSE);
}

boolean misc_color_pick(int x, int y, DragStatusType status)
{
	GShandlelist(colorList, x, y, status);
	return TRUE;
}

boolean misc_color_scroll(int x, int y, DragStatusType status)
{
	GShandlesclist(colorScroll, x, y, status);
	return TRUE;
}

static char *GetColor(int curColor)
{
	if (curColor < numListColors)
		return IEColorlist[curColor];
	else if (curColor < numColors)
		return IEAlias[curColor - numListColors];
	else
		return "";
}

boolean misc_set_highlight()
{
	strcpy(iestate.highlight, GetColor(curColor));
	ShowButtonStuffFields(F_HIGHLIGHT);
}

boolean misc_set_border()
{
	strcpy(iestate.buttonborder, GetColor(curColor));
	ShowButtonStuffFields(F_BORDER);
}

boolean misc_set_tlcolor()
{
	strcpy(iestate.buttonTLcolor, GetColor(curColor));
	ShowButtonStuffFields(F_TL);
}

boolean misc_set_brcolor()
{
	strcpy(iestate.buttonBRcolor, GetColor(curColor));
	ShowButtonStuffFields(F_BR);
}

boolean misc_col_up()
{
	GShandleupbutton(colorScroll);
}

boolean misc_col_down()
{
	GShandledownbutton(colorScroll);
}
